package com.smcic.api.epg.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.api.epg.dto.DisableDTO;
import com.smcic.api.epg.entity.Channel;
import com.smcic.api.epg.entity.EpgDisableConfig;
import com.smcic.api.epg.entity.Program;
import com.smcic.api.epg.mapper.EpgDisableConfigMapper;
import com.smcic.api.epg.service.IEpgDisableConfigService;
import com.smcic.core.advice.APIException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

@Service
public class EpgDisableConfigService extends ServiceImpl<EpgDisableConfigMapper, EpgDisableConfig> implements IEpgDisableConfigService {

    @Resource
    private ChannelService channelService;

    @Resource
    private ProgramService programService;

    public Map<LocalDateTime, EpgDisableConfig> cfMap(Integer channelId, LocalDateTime start, LocalDateTime end){
        Map<LocalDateTime, EpgDisableConfig> map = new HashMap<>();
        lambdaQuery().eq(EpgDisableConfig::getChannelId, channelId).ge(EpgDisableConfig::getStartTime, start)
                .le(EpgDisableConfig::getStartTime, end).eq(EpgDisableConfig::getStatus, 1).list()
                .forEach(sspDisableConfig -> map.put(sspDisableConfig.getStartTime(), sspDisableConfig));
        return map;
    }

    public void disable(DisableDTO disableDTO){
        Channel channel = channelService.getBaseMapper().channelById(disableDTO.getChannelId());
        if(null == channel){
            throw new APIException("不存在的频率频道");
        }
        Program program = programService.getBaseMapper().byStartAndChannel(disableDTO.getChannelId(), disableDTO.getStartTime());

        if(null == program){
            throw new APIException("不存在的节目");
        }

        EpgDisableConfig epgDisableConfig = new EpgDisableConfig();
        epgDisableConfig.setChannelId(disableDTO.getChannelId());
        epgDisableConfig.setChannelName(channel.getTitle());
        epgDisableConfig.setStartTime(disableDTO.getStartTime());
        epgDisableConfig.setProgram(program.getTitle());
        epgDisableConfig.setStatus(1);
        save(epgDisableConfig);
        programService.setCache(disableDTO.getChannelId());
    }

    public void update(DisableDTO disableDTO){
        EpgDisableConfig edc = getById(disableDTO.getId());

        if(null == edc){
            throw new APIException("不存在的数据记录");
        }

        edc.setStatus(1);

        updateById(edc);

    }
}
