package com.smcic.core.conf;

import com.github.benmanes.caffeine.cache.Caffeine;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.caffeine.CaffeineCache;
import org.springframework.cache.support.SimpleCacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

/**
 * 缓存全局配置
 * <p>
 * Created by zyx on 2017/8/2.
 */
@EnableCaching
@Configuration
public class CacheConfig {

    private static final int DEFAULT_MAXSIZE = 50000;

    private static final int DEFAULT_TTL = 60 * 30;//缓存过期时间为半个小时


    public enum Caches {

        WJWAPI_REQUEST,
        SINAAPI_CHINAREQUEST;

        Caches() {
        }

        Caches(int ttl) {
            this.ttl = ttl;
        }

        Caches(int ttl, int maxSize) {
            this.ttl = ttl;
            this.maxSize = maxSize;
        }

        /**
         * 最大数量
         */
        private int maxSize = DEFAULT_MAXSIZE;
        /**
         * 过期时间 秒
         */
        private int ttl = DEFAULT_TTL;

        public int getMaxSize() {
            return maxSize;
        }

        public int getTtl() {
            return ttl;
        }

    }

    /**
     * 创建基于Caffeine的Cache Manager
     * spring boot 默认推荐使用缓存库
     */
    @Bean
    @Primary
    public CacheManager caffeineCacheManager() {
        SimpleCacheManager cacheManager = new SimpleCacheManager();
        ArrayList<CaffeineCache> caches = new ArrayList<>();
        for (Caches c : Caches.values()) {
            caches.add(new CaffeineCache(c.name(),
                    Caffeine.newBuilder().recordStats()
                            .expireAfterWrite(c.getTtl(), TimeUnit.SECONDS)
                            .maximumSize(c.getMaxSize())
                            .build())
            );
        }

        cacheManager.setCaches(caches);

        return cacheManager;
    }

}
